"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.untypeEvent = exports.typeEvent = void 0;

var _actions = require("./actions");

var typeId = '@@type';
var globalStore = {};
/**
 * Converts regular events in to typed events
 * Can be used with update functions that expect type events
 */

var typeEvent = function typeEvent(update) {
  var store = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : globalStore;
  var create = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : _actions.createAction;

  var map = function map(event) {
    if (!event.type) {
      return event;
    }

    var type = event.type,
        _event$payload = event.payload,
        payload = _event$payload === void 0 ? null : _event$payload;
    var typed = store[type];

    if (typed) {
      return typed.of(payload);
    }

    store[type] = create(type);
    return store[type].of(payload);
  };

  return function (state, event) {
    return update(state, map(event));
  };
};
/**
 * Converts typed events in to regular events
 * Can be used with update functions that expect regular events
 */


exports.typeEvent = typeEvent;

var untypeEvent = function untypeEvent(update) {
  var map = function map(event) {
    if (!event.join || !event[typeId]) {
      return event;
    }

    return {
      type: event[typeId],
      payload: event.join()
    };
  };

  return function (state, event) {
    return update(state, map(event));
  };
};

exports.untypeEvent = untypeEvent;